// eslint-disable-next-line no-undef
window.a = 5;

/* @preserve this comment */

/* delete this comment */

// and this one too

// eslint-disable-next-line no-undef
if (window.a < 3) {
  console.log(4);
}
