module.exports = {
  description: 'does not replace plugin options',
  options: {
    original: 'replaced',
    delimiters: ['', ''],
    sourcemap: true,
    sourceMap: true,
    include: '**/input.js',
    exclude: 'node_modules/**'
  }
};
